// ===============================================================
// udemo.UZDownloader: This class is designed to download files off a uz master server.
// Basically a non-buffered httpclient
// ===============================================================

class UZDownloader expands TcpLink;

var UZHandler Saver; //saves the stuff downloaded
var DownloadClient GUI; //the GUI for this.
var int TotalSize; //total size of the file.
var int Downloaded; //ammount of data downloaded so far.
var float ElapsedTime; //time taken since start of binary data
var string ServerAddr; //server to download from
var string ServerURI; //local directory and file on server.
var string CF;  //carriage return (chr(13)) + line feed (chr(10))
var bool bHeaderRead;  //had http header been read?
var bool bResponded; //if got response
var string Line; //line of data
var bool bReadyToSave; //will save next tick (allow window time to update)

function PostBeginPlay()
{
  CF = Chr(13)$Chr(10);
  Super.PostBeginPlay();
}

event Destroyed(){
  close(); //dunno if this needed.
  Saver=none;
}
//handle errors by code
function setError(int code){
    //notification
    GUI.setError(code);
    destroy();
}

function Setup(string masterServer, string FName, GUID reqGUID){
  local int i;
  Saver = new (none) class'UZHandler';
  Saver.FileName = FName;
  Saver.FileGUID = reqGUID;
  i = instr(masterServer,"/");
  if (i!=-1){
     ServerURI = mid(masterServer,i);
     ServerAddr=left(masterServer,i);
  }
  else
    ServerAddr=masterServer;
  ServerURI = ServerURI$"/"$FName;
  Resolve(ServerAddr);
}

function ResolveFailed()
{
  setError(-3);
}

function Resolved( IpAddr Addr ){
//  xlog("resolved as"@Addr.Addr$":"$Addr.Port$"! Opening ip!");
  GUI.Success(0);
  Addr.Port=80; //force port 80 (HTTP)
  if (Addr.Addr==0){
      SetError(-3);
      log ("Invalid server address?");
  }
  if( BindPort() <= 0){
     SetError(-2);
     log("FAILED TO bind port");
     return;
  }

  Open(Addr);
}

event Opened()
{
  SendText("GET"@ServerURI@"HTTP/1.1"$cf$"Connection: close"$cf$"Host:"@ServerAddr$":80"$cf$cf); //send crap
  GUI.Success(1);
  LinkMode=MODE_Binary; //force!
  ReceiveMode=RMODE_Manual;
  enable('tick');
}

event Closed(){
  if (Saver!=none&&Downloaded>0) //valid save...
    bReadyToSave=true;
    GUI.Success(2);
  }
}

event ReceivedLine( string Line ){    //manual call
  local int i, res;
  if (!bResponded){
    i = InStr(Line, " ");
    res = Int(Mid(Line, i+1));
    if(res != 200) //200 = ok
      SetError(res);
    return;
  }
  if (Line==""){ //header done
    bHeaderRead=true;
    return;
  }
  if (left(Line,16)=="Content-Length: ") //size
     TotalSize = int(mid(Line,16));
}

function AddToLine (int Count, byte B[255]){
  local int i, j;
  for (i=0;i<Count;i++){
      if (bHeaderRead){ //all headers done.. send extra data as binary!
         while (i<Count){
           B[j]=B[i];
           j++;
           i++;
         }
         ReceivedBinary(j,B);
         return;
      }
      if (B[i] == 10 && asc(right(Line,1)) == 13){ //new line
          ReceivedLine(left(Line,len(line)-1));
          Line="";
          continue;
      }
      Line=Line$Chr(B[i]);
  }
}

function Tick(float Delta){ //manual read.. auto is bad.
  local byte B[255];
  local int i;
  if (bReadyToSave){   //allowed 1 tick for repainting.. now save.
     Saver.SaveFile(!bool(class'demosettings'.default.DownloadType));  //REMEMBER TO NOTIFY W/ RETVAL!
     destroy();
  }
  if (bHeaderRead)
    ElapsedTime+=Delta;
  while (IsDataPending()){
    i=ReadBinary(255,B);
    if (!bHeaderRead){
       AddToLine(i,B);
       continue;
    }
    if (i<=0){
      return;
    }
    ReceivedBinary(i,B);
  }
}

event ReceivedBinary( int Count, byte B[255] ){
  Saver.Append(B,Count);
  Downloaded+=Count;
  if (TotalSize!=-1&&Downloaded>=TotalSize){
    Close();
  }
  SetTimer(60.0, false);
}

function Timer(){ //time out...
  SetError(-1);
  xlog ("Time out. Still connected?"@(!IsConnected() || !Close()));
}

defaultproperties {
     ReceiveMode=RMODE_Event
     LinkMode=MODE_Binary
     TotalSize=-1
}

